@@ -9,6 +9,8 @@
 -- If there are any errors, it will let you know what and where
 -- they are, otherwise it will keep quiet.
 --
+-- Upgrading from a previous release? Check https://prosody.im/doc/upgrading
+--
 -- The only thing left to do is rename this file to remove the .dist ending, and fill in the
 -- blanks. Good luck, and happy Jabbering!
 
@@ -23,13 +25,6 @@
 -- Example: admins = { "user1@example.com", "user2@example.net" }
 admins = { }
 
--- Drop privileges
-prosody_user = "_prosody"
-prosody_group = "_prosody"
-
--- Enable POSIX-only options
-pidfile = "/var/prosody/prosody.pid"
-
 -- This option allows you to specify additional locations where Prosody
 -- will search first for modules. For additional modules you can install, see
 -- the community module repository at https://modules.prosody.im/
@@ -58,6 +53,8 @@ modules_enabled = {
 		"vcard_legacy"; -- Conversion between legacy vCard and PEP Avatar, vcard
 
 	-- Nice to have
+		"account_activity"; -- Record time when an account was last used
+		"cloud_notify"; -- Push notifications for mobile devices
 		"csi_simple"; -- Simple but effective traffic optimizations for mobile devices
 		"invites"; -- Create and manage invites
 		"invites_adhoc"; -- Allow admins/users to create invitations via their client
@@ -82,7 +79,6 @@ modules_enabled = {
 	-- Other specific functionality
 		--"announce"; -- Send announcement to all online users
 		--"groups"; -- Shared roster support
-		--"legacyauth"; -- Legacy authentication. Only used by some old clients and bots.
 		--"mimicking"; -- Prevent address spoofing
 		--"motd"; -- Send a message to users when they log in
 		--"proxy65"; -- Enables a file transfer proxy service which clients behind NAT can use
@@ -99,7 +95,6 @@ modules_disabled = {
 	-- "offline"; -- Store offline messages
 	-- "c2s"; -- Handle client connections
 	-- "s2s"; -- Handle server-to-server connections
-	-- "posix"; -- POSIX functionality, sends server to background, etc.
 }
 
 
@@ -190,8 +185,8 @@ archive_expires_after = "1w" -- Remove archived messages after 1 week
 -- Logging configuration
 -- For advanced logging see https://prosody.im/doc/logging
 log = {
-	info = "/var/prosody/prosody.log"; -- Change 'info' to 'debug' for verbose logging
-	error = "/var/prosody/prosody.err";
+	info = "/var/log/prosody/prosody.log"; -- Change 'info' to 'debug' for verbose logging
+	error = "/var/log/prosody/prosody.err";
 	-- "*syslog"; -- Uncomment this for logging to syslog
 	-- "*console"; -- Log to the console, useful for debugging when running in the foreground
 }
@@ -212,6 +207,9 @@ log = {
 -- Location of directory to find certificates in (relative to main config file):
 certificates = "certs"
 
+-- Enable PID file by default
+pidfile = "/var/run/prosody/prosody.pid";
+
 ----------- Virtual hosts -----------
 -- You need to add a VirtualHost entry for each domain you wish Prosody to serve.
 -- Settings under each VirtualHost entry apply *only* to that host.
